<?php
/**
 * @package    JMyLife
 * @subpackage Components
 * @version    1.0.16
 * @copyright  2009-2010 Jeff Channell
 * administrator/components/com_jmylife/helpers/string.php
 * @license

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 * @since     1.0.14
 */
// no direct access
defined('_JEXEC') or die('Restricted access');
jimport('joomla.filesystem.file');
if (!class_exists('JmylifeConfiguration')) {
	$configfile = JPATH_ROOT.DS.'components'.DS.'com_jmylife'.DS.'libraries'.DS.'configuration.php';
	if( JFile::exists( $configfile ) ) require_once( $configfile );
}

class JmylifeStringHelper
{
	private static $_instance;
	private $cfg;

	function __construct() {
		$this->cfg = JmylifeConfiguration::getInstance();
	}

	function getInstance() {
		if (!self::$_instance) {
			self::$_instance = new JmylifeStringHelper();
		}
		return self::$_instance;
	}

	function __call($name, $args=false) {
		// for conversions
		$reg = '`^to(.*)$`';
		if (preg_match($reg, $name)) {
			$method = preg_replace($reg, '$1', $name);
			return $this->to(strtolower($method), $args[0]);
		}
	}

	function to($type, $string) {
		$filter = JFilterInput::getInstance(null, null, 1, 1);
		switch (strtolower($type)) {
			default:
				return $filter->clean($string, $type);
		}
	}

	/**
	 * breakLongWords
	 * @param  string
	 * @return string
	 */
	function breakLongWords($str) {
		// split by spaces
		$parts = explode(' ', $str);
		// walk
		for ($j=0; $j<count($parts); $j++) {
			// arbitrary? also don't inject side html parameters as of 1.0.9
			if (20 < strlen($parts[$j]) && !preg_match('`https?\:\/{2}`', $parts[$j])) {
				$parts[$j] = preg_replace('`([^&][a-zA-Z0-9]{8})`', "$1&#8203;", $parts[$j]);
			}
		}
		// put it back together
		return preg_replace('`([^&#8203]);`', '$1;&#8203;', implode(' ', $parts));
	}

	/**
	 * shorten
	 * @param   string
	 * @param   int
	 * @param   bool, default true
	 */
	function shorten($text, $count, $addbb=true) {
		if ($count) {
			$totalchars = strlen($text);
			$charcount = 0;
			$fulltext = $text;
			$buffer = '';
			if ($totalchars >= $count) {
				$bbcode = false;
				while (0 < strlen($fulltext)) {
					$char = $fulltext{0};
					if ($char == '[') {
						$bbcode = true;
					}
					if (($bbcode && $addbb ) || $charcount < $count) {
						$buffer .= $char;
					}
					$fulltext = substr($fulltext, 1);
					if (!$bbcode) {
						$charcount++;
					}
					if ($char == ']') {
						$bbcode = false;
					}
				}
				$text = $buffer.($charcount >= $count ? '...' : '');
			}
		}
		// fix for empty bb tags
		$text = str_replace('[li][/li]', '', $text);
		return $text;
	}

	/**
	 * stubs
	 * @param   string
	 * @param   int
	 * @param   string
	 * @return  string
	 */
	function stubs($text, $id, $type='story') {
		JLoader::import('components.com_jmylife.helpers.comment', JPATH_ADMINISTRATOR, '');
		$html =  JmylifeHtmlHelper::getInstance();
		$appl =& JFactory::getApplication();
		$user =& JFactory::getUser();
		$stubs = array(
			'{JMLSiteLink}' => $html->link(JURI::root(), $this->cfg->get('jmylife_name', 'JMyLife')),
			'{JMLSiteURL}'  => JURI::root(),
			'{JMLSite}'     => $this->cfg->get('jmylife_name', 'JMyLife'),
			'{JMLUser}'     => $user->guest ? 'guest' : $user->username,
			'{JMLUserID}'   => $user->id
		);
		switch ($type) {
			case 'comment':
				JLoader::import('components.com_jmylife.helpers.comment', JPATH_ADMINISTRATOR, '');
				$helper = JmylifeCommentHelper::getInstance();
				// use id for comment this time
				$query = $helper->buildQueryById($id);
				$this->cfg->db->setQuery($query);
				if ($comment = $this->cfg->db->loadObject()) {
					$comment = $helper->prepare($comment);
					$stubs['{JMLComment}']       = $comment->message;
					$stubs['{JMLCommentAuthor}'] = $comment->author;
					// set $id to the comment's sid
					$id = $comment->sid;
					// don't break after this, so story stubs get parsed too!
				}
			case 'story':
				// use id to get story
				JLoader::import('components.com_jmylife.helpers.story', JPATH_ADMINISTRATOR, '');
				$helper = JmylifeStoryHelper::getInstance();
				$query = $helper->buildQuery(array('view' => 'item', 'id' => $id), -1);
				$this->cfg->db->setQuery($query);
				if ($story = $this->cfg->db->loadObject()) {
					$story     = $helper->prepare($story, 'item', false);
					$storylink = preg_replace('`\&amp\;`', '&', $story->fullstorylink);
					$catlink   = preg_replace('`\&amp\;`', '&', $story->fullcatlink);
					$title     = 0 < strlen($story->title) ? $story->title : JText::sprintf('COM_JMYLIFE_STORY_TITLE', $story->user_alias);
					$stubs['{JMLStory}']             = $story->fulltext;
					$stubs['{JMLStoryAuthor}']       = $story->user_alias;
					$stubs['{JMLStoryTitle}']        = $title;
					$stubs['{JMLStoryCategory}']     = $story->catname;
					$stubs['{JMLStoryCategoryURL}']  = $catlink;
					$stubs['{JMLStoryCategoryLink}'] = '<a href="'.$catlink.'">'.$story->catname.'</a>';
					$stubs['{JMLStoryURL}']          = $storylink;
					$stubs['{JMLStoryLink}']         = '<a href="'.$storylink.'">'.$title.'</a>';
				}
				break;
		}
		return str_replace(array_keys($stubs), $stubs, $text);
	}

}